% maxsymsep   Maximum for symmetric product states. Uses simple
%   search for which parameters can be set in the beginning of the
%   maxsymsep.m file. Faster than maxsep.

% Copyright (C) 2005  Geza Toth    E.mail: toth@alumni.nd.edu
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; see gpl.txt
% of this subroutine package.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.

function m=maxsymsep(op)

[sx,sy]=size(op);
N=log2(sx);

% Parameters for the simulation
Delta=0.005;
Nit1=1000;
Nit2=2000;

rmax=-Inf;

% Dimesnions
d=2;

fa=zeros(d,N);
famax=fa;
for n=1:Nit1
     if mod(n,100)==0,  randn('state',sum(100*clock));  end %if
     f=(rand(d,1)-0.5)+i*(rand(d,1)-0.5);
     fa=f;
     for n=1:N-1
        f=kron(f,fa);
     end %for
     r=real(trace(op*f*f')/(f'*f));
     if r>rmax,
         rmax=r;
	 famax=fa;
     end %if
end %for


fa0=famax;
r0=rmax;

% Second phase of the search
for n=1:Nit2
     if mod(n,100)==0,  randn('state',sum(100*clock));  end %if
     f=(rand(d,1)-0.5)+i*(rand(d,1)-0.5);
     fa=fa0+Delta*f;
     for n=1:N-1
        f=kron(f,fa);
     end %for
     r=real(trace(op*f*f')/(f'*f));
     if r>r0,
         r0=r;
         fa0=fa;        
     end %if
end %for
 
m=r0;

